#include "liblocky.h"

#include <assert.h>


bool blky_decoder_feed(blky_decoder_t* de, uint32_t block_index) {
  assert(de->block_num > 0);
  assert(de->feat_bits > 0);
  assert(de->seed      > 0);

  const uint64_t seed = blky_numeric_xorshift64(de->seed);

  if (de->count++) {
    const uint32_t feat_max = 1 << de->feat_bits;
    assert(feat_max < de->block_num);

    uint32_t feat = 0;
    for (; feat < feat_max; ++feat) {
      if (blky_numeric_hop(de->block_index, feat, seed)%de->block_num == block_index) {
        break;
      }
    }
    if (feat >= feat_max) return false;

    assert(de->scrap_bits+8 <= 32);
    de->scrap      |= feat << de->scrap_bits;
    de->scrap_bits += de->feat_bits;
  }

  de->seed        = seed;
  de->block_index = block_index;
  return true;
}

bool blky_decoder_pop(blky_decoder_t* de, uint8_t* b, bool force) {
  if (force) {
    if (de->scrap_bits > 0) return false;
  } else {
    if (de->scrap_bits < 8) return false;
  }

  *b = (uint8_t) (de->scrap & 0xFF);

  de->scrap >>= 8;
  if (de->scrap_bits >= 8) {
    de->scrap_bits -= 8;
  } else {
    de->scrap_bits = 0;
  }
  return true;
}
